//! Handle requests for cover images.

use std::{io::Cursor, num::NonZeroU32, sync::Arc};

use crate::{app_state::AppState, handlers::error::HandlerError};
use fast_image_resize::{Image, PixelType, ResizeAlg, Resizer};
use image::{codecs::jpeg::JpegEncoder, io::Reader as ImageReader, ColorType, ImageEncoder};
use poem::{
    error::NotFoundError,
    handler,
    web::{headers::ContentType, Data, Path},
    Response,
};
use tokio::fs::File;

/// Handle a request for the cover image of a book with id `id`.
#[handler]
pub async fn handler(id: Path<u64>, state: Data<&Arc<AppState>>) -> Result<Response, poem::Error> {
    let book = state
        .calibre
        .scalar_book(*id)
        .map_err(HandlerError::DataError)?;
    let cover_path = state.config.library_path.join(book.path).join("cover.jpg");
    let mut cover = File::open(cover_path).await.map_err(|_| NotFoundError)?;
    crate::handlers::download::handler("cover.jpg", cover, &ContentType::jpeg().to_string()).await
}

/// Handle a request for the cover thumbnail of a book with id `id`.
#[handler]
pub async fn handler_thumbnail(
    id: Path<u64>,
    state: Data<&Arc<AppState>>,
) -> Result<Response, poem::Error> {
    let book = state
        .calibre
        .scalar_book(*id)
        .map_err(HandlerError::DataError)?;
    let cover_path = state.config.library_path.join(book.path).join("cover.jpg");

    let img = ImageReader::open(cover_path)
        .map_err(HandlerError::IoError)?
        .decode()
        .map_err(HandlerError::ImageError)?;

    let width = NonZeroU32::new(img.width()).ok_or(HandlerError::ImageSizeError)?;
    let height = NonZeroU32::new(img.height()).ok_or(HandlerError::ImageSizeError)?;

    let mut src_image =
        Image::from_vec_u8(width, height, img.to_rgb8().into_raw(), PixelType::U8x3)
            .map_err(HandlerError::ImageBufferError)?;

    let dst_width = NonZeroU32::new(350).ok_or(HandlerError::ImageSizeError)?;
    let dst_height = NonZeroU32::new(525).ok_or(HandlerError::ImageSizeError)?;
    let mut dst_image = Image::new(dst_width, dst_height, src_image.pixel_type());
    let mut dst_view = dst_image.view_mut();

    let mut resizer = Resizer::new(ResizeAlg::Nearest);
    resizer
        .resize(&src_image.view(), &mut dst_view)
        .map_err(HandlerError::PixelTypeError)?;

    let mut resized_img = Cursor::new(Vec::new());
    JpegEncoder::new(&mut resized_img)
        .write_image(
            dst_image.buffer(),
            dst_width.get(),
            dst_height.get(),
            ColorType::Rgb8.into(),
        )
        .map_err(HandlerError::ImageError)?;

    resized_img.set_position(0);

    crate::handlers::download::handler("cover.jpg", resized_img, &ContentType::jpeg().to_string())
        .await
}
